@@ -8,7 +8,7 @@
 
 #include <asm/types.h>          /* For __uXX types */
 
-#define IP_VS_VERSION_CODE            0x00090A
+#define IP_VS_VERSION_CODE            0x010000
 #define NVERSION(version)                       \
 	(version >> 16) & 0xFF,                 \
 	(version >> 8) & 0xFF,                  \
@@ -325,7 +325,6 @@ enum {
 	NET_IPV4_VS_TO_ICMP=18,
 	NET_IPV4_VS_LBLC_EXPIRE=19,
 	NET_IPV4_VS_LBLCR_EXPIRE=20,
-	NET_IPV4_VS_USE_DST=21,
 	NET_IPV4_VS_CACHE_BYPASS=22,
 	NET_IPV4_VS_EXPIRE_NODEST_CONN=23,
 	NET_IPV4_VS_SYNC_THRESHOLD=24,
@@ -700,7 +699,6 @@ extern void ip_vs_scheduler_put(struct ip_vs_scheduler *scheduler);
  *      IPVS control data and functions
  *      (from ip_vs_ctl.c)
  */
-extern int sysctl_use_dst;
 extern int sysctl_ip_vs_cache_bypass;
 extern int sysctl_ip_vs_expire_nodest_conn;
 extern int sysctl_ip_vs_sync_threshold;
@@ -873,9 +871,8 @@ __ip_vs_get_out_rt(struct ip_vs_conn *cp, u32 rtos)
 
 	if (dest) {
 		spin_lock(&dest->dst_lock);
-		if (!sysctl_use_dst
-		    || !(rt = (struct rtable *)
-			 __ip_vs_dst_check(dest, rtos, 0))) {
+		if (!(rt = (struct rtable *)
+		      __ip_vs_dst_check(dest, rtos, 0))) {
 			if (ip_route_output(&rt, dest->addr, 0, rtos, 0)) {
 				spin_unlock(&dest->dst_lock);
 				IP_VS_DBG_RL("ip_route_output error, "
