@@ -5,11 +5,11 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_conn.c,v 1.24 2002/01/24 14:23:48 wensong Exp $
+ * Version:     $Id: ip_vs_conn.c,v 1.25 2002/01/31 14:33:12 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *              Peter Kese <peter.kese@ijs.si>
- *              Julian Anastasov <uli@linux.tu-varna.acad.bg>
+ *              Julian Anastasov <ja@ssi.bg>
  *
  *              This program is free software; you can redistribute it and/or
  *              modify it under the terms of the GNU General Public License
@@ -29,6 +29,7 @@
 #include <linux/types.h>
 #include <linux/kernel.h>
 #include <linux/errno.h>
+#include <linux/compiler.h>
 #include <linux/vmalloc.h>
 #include <linux/ip.h>
 #include <linux/tcp.h>                  /* for tcphdr */
@@ -110,6 +111,16 @@ static inline void ct_read_unlock_bh(unsigned key)
 	read_unlock_bh(&__ip_vs_conntbl_lock_array[key&CT_LOCKARRAY_MASK].l);
 }
 
+static inline void ct_write_lock_bh(unsigned key)
+{
+	write_lock_bh(&__ip_vs_conntbl_lock_array[key&CT_LOCKARRAY_MASK].l);
+}
+
+static inline void ct_write_unlock_bh(unsigned key)
+{
+	write_unlock_bh(&__ip_vs_conntbl_lock_array[key&CT_LOCKARRAY_MASK].l);
+}
+
 
 /*
  *	Returns hash value for IPVS connection entry
@@ -128,7 +139,7 @@ ip_vs_conn_hashkey(unsigned proto, __u32 addr, __u16 port)
  *	Hashes ip_vs_conn in ip_vs_conn_tab by proto,addr,port.
  *	returns bool success.
  */
-int ip_vs_conn_hash(struct ip_vs_conn *cp)
+static int ip_vs_conn_hash(struct ip_vs_conn *cp)
 {
 	unsigned hash;
 
@@ -157,7 +168,7 @@ int ip_vs_conn_hash(struct ip_vs_conn *cp)
  *	UNhashes ip_vs_conn from ip_vs_conn_tab.
  *	returns bool success.
  */
-int ip_vs_conn_unhash(struct ip_vs_conn *cp)
+static int ip_vs_conn_unhash(struct ip_vs_conn *cp)
 {
 	unsigned hash;
 
@@ -611,8 +622,8 @@ static int ip_vs_bypass_xmit(struct sk_buff *skb, struct ip_vs_conn *cp)
 	/* MTU checking */
 	mtu = rt->u.dst.pmtu;
 	if ((skb->len > mtu) && (iph->frag_off&__constant_htons(IP_DF))) {
-		icmp_send(skb, ICMP_DEST_UNREACH,ICMP_FRAG_NEEDED, htonl(mtu));
 		ip_rt_put(rt);
+		icmp_send(skb, ICMP_DEST_UNREACH,ICMP_FRAG_NEEDED, htonl(mtu));
 		IP_VS_DBG_RL("ip_vs_bypass_xmit(): frag needed\n");
 		goto tx_error;
 	}
@@ -719,8 +730,8 @@ static int ip_vs_nat_xmit(struct sk_buff *skb, struct ip_vs_conn *cp)
 	/* MTU checking */
 	mtu = rt->u.dst.pmtu;
 	if ((skb->len > mtu) && (iph->frag_off&__constant_htons(IP_DF))) {
-		icmp_send(skb, ICMP_DEST_UNREACH,ICMP_FRAG_NEEDED, htonl(mtu));
 		ip_rt_put(rt);
+		icmp_send(skb, ICMP_DEST_UNREACH,ICMP_FRAG_NEEDED, htonl(mtu));
 		IP_VS_DBG_RL("ip_vs_nat_xmit(): frag needed\n");
 		goto tx_error;
 	}
@@ -1174,7 +1185,7 @@ static void ip_vs_conn_expire(unsigned long data)
 	/*
 	 *	refcnt==1 implies I'm the only one referrer
 	 */
-	if (atomic_read(&cp->refcnt) == 1) {
+	if (likely(atomic_read(&cp->refcnt) == 1)) {
 		/* make sure that there is no timer on it now */
 		if (timer_pending(&cp->timer))
 			del_sltimer(&cp->timer);
@@ -1191,20 +1202,28 @@ static void ip_vs_conn_expire(unsigned long data)
 		atomic_dec(&ip_vs_conn_count);
 
 		kmem_cache_free(ip_vs_conn_cachep, cp);
-		goto expire_out;
+		return;
 	}
 
+	/* hash it back to the table */
+	ip_vs_conn_hash(cp);
+
   expire_later:
 	IP_VS_DBG(7, "delayed: refcnt-1=%d conn.n_control=%d\n",
 		  atomic_read(&cp->refcnt)-1,
 		  atomic_read(&cp->n_control));
 
 	ip_vs_conn_put(cp);
-
-  expire_out:
 }
 
 
+void ip_vs_conn_expire_now(struct ip_vs_conn *cp)
+{
+	cp->timeout = 0;
+	mod_sltimer(&cp->timer, jiffies);
+	__ip_vs_conn_put(cp);
+}
+
 /*
  *  Create a new connection entry and hash it into the ip_vs_conn_tab.
  */
@@ -1405,16 +1424,16 @@ void ip_vs_random_dropentry(void)
 			/*
 			 * Drop the entry, and drop its ct if not referenced
 			 */
+			atomic_inc(&cp->refcnt);
 			ct_write_unlock(hash);
-			IP_VS_DBG(4, "Drop connection\n");
-			if (del_sltimer(&cp->timer)) {
-				ct = cp->control;
-				ip_vs_conn_expire_now(cp);
-				if (ct && !atomic_read(&ct->n_control)) {
-					IP_VS_DBG(4, "Drop conn template\n");
-					if(del_sltimer(&ct->timer))
-						ip_vs_conn_expire_now(ct);
-				}
+
+			if ((ct = cp->control))
+				atomic_inc(&ct->refcnt);
+			IP_VS_DBG(4, "del connection\n");
+			ip_vs_conn_expire_now(cp);
+			if (ct) {
+				IP_VS_DBG(4, "del conn template\n");
+				ip_vs_conn_expire_now(ct);
 			}
 			ct_write_lock(hash);
 		}
@@ -1433,31 +1452,30 @@ static void ip_vs_conn_flush(void)
 	struct list_head *l,*e;
 	struct ip_vs_conn *ct;
 
- flush_again:
+  flush_again:
 	for (idx=0; idx<IP_VS_CONN_TAB_SIZE; idx++) {
 		/*
 		 *  Lock is actually needed in this loop.
 		 */
-		ct_write_lock(idx);
+		ct_write_lock_bh(idx);
 
 		l = &ip_vs_conn_tab[idx];
 		for (e=l->next; e!=l; e=e->next) {
 			cp = list_entry(e, struct ip_vs_conn, c_list);
+			atomic_inc(&cp->refcnt);
 			ct_write_unlock(idx);
-			IP_VS_DBG(4, "del connection\n");
-			if (del_sltimer(&cp->timer)) {
-				ct = cp->control;
-				ip_vs_conn_expire_now(cp);
 
-				if (ct && !atomic_read(&ct->n_control)) {
-					IP_VS_DBG(4, "del conn template\n");
-					if(del_sltimer(&ct->timer))
-						ip_vs_conn_expire_now(ct);
-				}
+			if ((ct = cp->control))
+				atomic_inc(&ct->refcnt);
+			IP_VS_DBG(4, "del connection\n");
+			ip_vs_conn_expire_now(cp);
+			if (ct) {
+				IP_VS_DBG(4, "del conn template\n");
+				ip_vs_conn_expire_now(ct);
 			}
 			ct_write_lock(idx);
 		}
-		ct_write_unlock(idx);
+		ct_write_unlock_bh(idx);
 	}
 
 	/* the counter may be not NULL, because maybe some conn entries
