@@ -5,7 +5,7 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_ctl.c,v 1.25 2002/01/31 14:33:12 wensong Exp $
+ * Version:     $Id: ip_vs_ctl.c,v 1.26 2002/02/27 02:50:18 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *              Peter Kese <peter.kese@ijs.si>
@@ -76,7 +76,6 @@ static int sysctl_ip_vs_drop_packet = 0;
 static int sysctl_ip_vs_secure_tcp = 0;
 static int sysctl_ip_vs_amemthresh = 1024;
 static int sysctl_ip_vs_am_droprate = 10;
-int sysctl_use_dst = 1;
 int sysctl_ip_vs_cache_bypass = 0;
 int sysctl_ip_vs_expire_nodest_conn = 0;
 int sysctl_ip_vs_sync_threshold = 3;
@@ -871,7 +870,6 @@ static int ip_vs_edit_dest(struct ip_vs_service *svc,
 /*
  *  Delete a destination (must be already unlinked from the service)
  */
-
 static void __ip_vs_del_dest(struct ip_vs_dest *dest)
 {
 	ip_vs_kill_estimator(&dest->stats);
@@ -1353,9 +1351,6 @@ static struct ip_vs_sysctl_table ipv4_vs_table = {
 	  &sysctl_ip_vs_debug_level, sizeof(int), 0644, NULL,
 	  &proc_dointvec},
 #endif
-	 {NET_IPV4_VS_USE_DST, "use_dst",
-	  &sysctl_use_dst, sizeof(int), 0644, NULL,
-	  &proc_dointvec},
 	 {NET_IPV4_VS_AMDROPRATE, "am_droprate",
 	  &sysctl_ip_vs_am_droprate, sizeof(int), 0644, NULL,
 	  &proc_dointvec},
