@@ -1,7 +1,7 @@
 /*
  * IPVS:        Locality-Based Least-Connection scheduling module
  *
- * Version:     $Id: ip_vs_lblc.c,v 1.7 2001/10/19 15:05:17 wensong Exp $
+ * Version:     $Id: ip_vs_lblc.c,v 1.8 2001/12/11 12:41:29 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@gnuchina.org>
  *
@@ -348,7 +348,7 @@ static void ip_vs_lblc_check_expire(unsigned long data)
 		goto out;
 	}
 
-	if (atomic_read(&tbl->entries) < tbl->max_size) {
+	if (atomic_read(&tbl->entries) <= tbl->max_size) {
 		tbl->counter++;
 		goto out;
 	}
